@@ -18,6 +18,7 @@ import copy
 from functools import reduce
 import rnn
 from pruningModel import pruningNoSenseTensor
+from fuseOps import opListFuse
 
 
 # 输入模型所在目录
@@ -35,7 +36,7 @@ sliceDataSize = 4 * 1024
 # paddlepaddle运行程序实例
 program = None
 # 存放模型结构
-modelInfo = {"vars": {}, "ops": [], "chunkNum": 0, "dataLayout": "nhwc", "feedShape": None}
+modelInfo = {"vars": {}, "ops": [], "chunkNum": 0, "dataLayout": "nchw", "feedShape": None}
 # 存放参数数值（未排序）
 paramValuesDict = {}
 
@@ -193,12 +194,6 @@ def organizeModelVariableInfo(result):
         # persistable数据存入paramValuesDict，等待排序
         if v.persistable:
             tensor = np.array(fluid.global_scope().find_var(v.name).get_tensor())
-            # nchw 转 nhwc
-            if len(tensor.shape) == 3:
-                tensor = np.transpose(tensor, (1, 2, 0))
-            elif len(tensor.shape) == 4:
-                tensor = np.transpose(tensor, (0, 2, 3, 1))
-
             data = tensor.flatten().tolist()
             paramValuesDict[v.name] = data
 
@@ -273,7 +268,7 @@ def organizeModelOpInfo():
         # 获取OP output
         outputs = {}
         # 将outputs转换为数组
-        if op.type == 'density_prior_box' or op.type == 'prior_box' or op.type == 'box_coder' or op.type == 'box_coder':
+        if op.type == 'density_prior_box' or op.type == 'prior_box' or op.type == 'box_coder':
             outputs['Out'] = []
             for name in opOutputs:
                 value = op.output(name)
@@ -294,6 +289,18 @@ def organizeModelOpInfo():
 
         opInfo["outputs"] = outputs
 
+        # 收敛outputs[name]
+        if "Output" in opInfo["outputs"]:
+            opInfo["outputs"]["Out"] = opInfo["outputs"]["Output"]
+            del opInfo["outputs"]["Output"]
+
+        elif "Y" in opInfo["outputs"]:
+            opInfo["outputs"]["Out"] = opInfo["outputs"]["Y"]
+            del opInfo["outputs"]["Y"]
+
+        if "Out" not in opInfo["outputs"]:
+            print("\033[31moutputs[name] not exist Out.\033[0m")
+            sys.exit(1)
 
         # 有的模型如人脸关键点，会出现两个算子合并的情况，如lmk_demo，elementwise_add后接了relu算子，relu的输入输出相等，兼容一下
         # inputs与outputs只有一个，名称相等，则，输入加后缀，改上一层算子。
@@ -328,7 +335,7 @@ def organizeModelOpInfo():
             rnnList.append(index)
 
         # multiclass_nms 单独处理
-        if (op.type == 'multiclass_nms'):
+        if (op.type.startswith('multiclass_nms')):
             postOps.append(opInfo)
         else:
             # 存入modelInfo
@@ -340,7 +347,6 @@ def organizeModelOpInfo():
         index += 1
     print("Organizing model operators info successfully.")
 
-
 def addChunkNumToJson(paramValueList):
     totalParamValuesCount = len(paramValueList)
     countPerSlice = int(sliceDataSize * 1024 / 4)
@@ -409,8 +415,6 @@ def appendConnectOp(fetch_targets):
     modelInfo['multiOutputs'] = targets
     return targets
 
-
-
 def genModelFeedShape(feed):
     if len(feed) != 1:
         print("\033[33;1mModel has more than one input feed.\033[0m")
@@ -436,8 +440,7 @@ def genModelFeedShape(feed):
     modelInfo['feedShape'] = feedShape
     print("\033[32mModel FeedShape set successfully.\033[0m")
 
-
-def convertToPaddleJSModel(modelDir, modelName, paramsName, outputDir):
+def convertToPaddleJSModel(modelDir, modelName, paramsName, outputDir, useGPUOpt):
     """ 转换fluid modle为paddleJS model """
 
 
@@ -464,13 +467,18 @@ def convertToPaddleJSModel(modelDir, modelName, paramsName, outputDir):
         for index in rnnList:
             rnn.splice_rnn_op(modelInfo, index)
 
+    if useGPUOpt:
+        # 算子融合
+        modelInfo['gpuOpt'] = True
+        opListFuse(modelInfo['ops'])
+
     # 对多输出模型追加connect算子
     if len(fetch_targets) > 1:
         appendConnectOp(fetch_targets)
 
     if (postOps and len(postOps) > 0):
         for op in postOps:
-            if (op['type'] == 'multiclass_nms'):
+            if (op['type'].startswith('multiclass_nms')):
                 inputNames = []
                 for input, value in op['inputs'].items():
                     if len(value) <= 0:
@@ -505,8 +513,6 @@ def convertToPaddleJSModel(modelDir, modelName, paramsName, outputDir):
     # 导出分片参数文件
     sliceDataToBinaryFile(paramValues, outputDir)
 
-
-
 def main():
 
     global sliceDataSize
@@ -520,11 +526,13 @@ def main():
         p.add_argument("--outputDir", help='paddleJS模型输出路径，必要参数', required=True)
         p.add_argument("--logModelInfo", type=int, default=0, help='是否输出模型结构信息，非必要参数，0为不输出，1为输出，默认不输出', required=False)
         p.add_argument("--sliceDataSize", type=int, default=4096, help='分片输出参数文件时，每片文件的大小，单位：KB，非必要参数，默认4096KB', required=False)
+        p.add_argument('--useGPUOpt', help='转换模型是否执行GPU优化方法', required=False)
 
         args = p.parse_args()
         modelDir = args.inputDir
         modelPath = args.modelPath
         paramPath = args.paramPath
+        useGPUOpt = args.useGPUOpt
 
         if not modelDir:
             modelDir, modelName = os.path.split(modelPath)
@@ -538,7 +546,7 @@ def main():
         if args.logModelInfo == 1:
             enableLogModelInfo = True
 
-        convertToPaddleJSModel(modelDir, modelName, paramsName, outputDir)
+        convertToPaddleJSModel(modelDir, modelName, paramsName, outputDir, useGPUOpt)
 
     except Exception as identifier:
         print("\033[31mA fetal error occured. Failed to convert model.\033[0m")
@@ -546,4 +554,4 @@ def main():
         sys.exit(1)
 
 if __name__ == "__main__":
-    main()
\ No newline at end of file
+    main()
